--- linux-2.6.27.6/drivers/parport/parport_pc.c	2008-11-14 01:56:21.000000000 +0800
+++ linux-2.6.27.6.yaffs/drivers/parport/parport_pc.c	2011-05-08 18:00:38.000000000 +0800
@@ -120,7 +120,8 @@
 	DPRINTK (KERN_DEBUG "frob_econtrol(%02x,%02x): %02x -> %02x\n",
 		m, v, ectr, (ectr & ~m) ^ v);
 
-	outb ((ectr & ~m) ^ v, ECONTROL (pb));
+	//outb ((ectr & ~m) ^ v, ECONTROL (pb));
+	writeb ((ectr & ~m) ^ v, ECONTROL (pb));
 }
 
 static __inline__ void frob_set_mode (struct parport *p, int mode)
@@ -149,7 +150,8 @@
 	}
 
 	/* Bits <7:5> contain the mode. */
-	oecr = inb (ECONTROL (p));
+	//oecr = inb (ECONTROL (p));
+	oecr = readb (ECONTROL (p));
 	mode = (oecr >> 5) & 0x7;
 	if (mode == m) return 0;
 
@@ -163,14 +165,16 @@
 		case ECR_ECP: /* ECP Parallel Port mode */
 			/* Busy wait for 200us */
 			for (counter = 0; counter < 40; counter++) {
-				if (inb (ECONTROL (p)) & 0x01)
+				//if (inb (ECONTROL (p)) & 0x01)
+				if (readb (ECONTROL (p)) & 0x01)
 					break;
 				if (signal_pending (current)) break;
 				udelay (5);
 			}
 
 			/* Poll slowly. */
-			while (!(inb (ECONTROL (p)) & 0x01)) {
+			//while (!(inb (ECONTROL (p)) & 0x01)) {
+			while (!(readb (ECONTROL (p)) & 0x01)) {
 				if (time_after_eq (jiffies, expire))
 					/* The FIFO is stuck. */
 					return -EBUSY;
@@ -257,8 +261,10 @@
 	/* To clear timeout some chips require double read */
 	parport_pc_read_status(pb);
 	r = parport_pc_read_status(pb);
-	outb (r | 0x01, STATUS (pb)); /* Some reset by writing 1 */
-	outb (r & 0xfe, STATUS (pb)); /* Others by writing 0 */
+	//outb (r | 0x01, STATUS (pb)); /* Some reset by writing 1 */
+	writeb (r | 0x01, STATUS (pb)); /* Some reset by writing 1 */
+	//outb (r & 0xfe, STATUS (pb)); /* Others by writing 0 */
+	writeb (r & 0xfe, STATUS (pb)); /* Others by writing 0 */
 	r = parport_pc_read_status(pb);
 
 	return !(r & 0x01);
@@ -289,14 +295,16 @@
 	const struct parport_pc_private *priv = p->physport->private_data;
 	s->u.pc.ctr = priv->ctr;
 	if (priv->ecr)
-		s->u.pc.ecr = inb (ECONTROL (p));
+		//s->u.pc.ecr = inb (ECONTROL (p));
+		s->u.pc.ecr = readb (ECONTROL (p));
 }
 
 static void parport_pc_restore_state(struct parport *p, struct parport_state *s)
 {
 	struct parport_pc_private *priv = p->physport->private_data;
 	register unsigned char c = s->u.pc.ctr & priv->ctr_writable;
-	outb (c, CONTROL (p));
+	//outb (c, CONTROL (p));
+	writeb (c, CONTROL (p));
 	priv->ctr = c;
 	if (priv->ecr)
 		ECR_WRITE (p, s->u.pc.ecr);
@@ -316,7 +324,8 @@
 		 *  nFault is 0 if there is at least 1 byte in the Warp's FIFO
 		 *  pError is 1 if there are 16 bytes in the Warp's FIFO
 		 */
-		status = inb (STATUS (port));
+		//status = inb (STATUS (port));
+		status = readb (STATUS (port));
 
 		while (!(status & 0x08) && (got < length)) {
 			if ((left >= 16) && (status & 0x20) && !(status & 0x08)) {
@@ -331,12 +340,14 @@
 				left -= 16;
 			} else {
 				/* grab single byte from the warp fifo */
-				*((char *)buf) = inb (EPPDATA (port));
+				//*((char *)buf) = inb (EPPDATA (port));
+				*((char *)buf) = readb (EPPDATA (port));
 				buf++;
 				got++;
 				left--;
 			}
-			status = inb (STATUS (port));
+			//status = inb (STATUS (port));
+			status = readb (STATUS (port));
 			if (status & 0x01) {
 				/* EPP timeout should never occur... */
 				printk (KERN_DEBUG "%s: EPP timeout occurred while talking to "
@@ -352,16 +363,19 @@
 		} else {
 			insb (EPPDATA (port), buf, length);
 		}
-		if (inb (STATUS (port)) & 0x01) {
+		//if (inb (STATUS (port)) & 0x01) {
+		if (readb (STATUS (port)) & 0x01) {
 			clear_epp_timeout (port);
 			return -EIO;
 		}
 		return length;
 	}
 	for (; got < length; got++) {
-		*((char*)buf) = inb (EPPDATA(port));
+		//*((char*)buf) = inb (EPPDATA(port));
+		*((char*)buf) = readb (EPPDATA(port));
 		buf++;
-		if (inb (STATUS (port)) & 0x01) {
+		//if (inb (STATUS (port)) & 0x01) {
+		if (readb (STATUS (port)) & 0x01) {
 			/* EPP timeout */
 			clear_epp_timeout (port);
 			break;
@@ -382,7 +396,8 @@
 		} else {
 			outsb (EPPDATA (port), buf, length);
 		}
-		if (inb (STATUS (port)) & 0x01) {
+		//if (inb (STATUS (port)) & 0x01) {
+		if (readb (STATUS (port)) & 0x01) {
 			clear_epp_timeout (port);
 			return -EIO;
 		}
@@ -391,7 +406,8 @@
 	for (; written < length; written++) {
 		outb (*((char*)buf), EPPDATA(port));
 		buf++;
-		if (inb (STATUS(port)) & 0x01) {
+		//if (inb (STATUS(port)) & 0x01) {
+		if (readb (STATUS(port)) & 0x01) {
 			clear_epp_timeout (port);
 			break;
 		}
@@ -407,16 +423,19 @@
 
 	if ((flags & PARPORT_EPP_FAST) && (length > 1)) {
 		insb (EPPADDR (port), buf, length);
-		if (inb (STATUS (port)) & 0x01) {
+		//if (inb (STATUS (port)) & 0x01) {
+		if (readb (STATUS (port)) & 0x01) {
 			clear_epp_timeout (port);
 			return -EIO;
 		}
 		return length;
 	}
 	for (; got < length; got++) {
-		*((char*)buf) = inb (EPPADDR (port));
+		//*((char*)buf) = inb (EPPADDR (port));
+		*((char*)buf) = readb (EPPADDR (port));
 		buf++;
-		if (inb (STATUS (port)) & 0x01) {
+		//if (inb (STATUS (port)) & 0x01) {
+		if (readb (STATUS (port)) & 0x01) {
 			clear_epp_timeout (port);
 			break;
 		}
@@ -433,7 +452,8 @@
 
 	if ((flags & PARPORT_EPP_FAST) && (length > 1)) {
 		outsb (EPPADDR (port), buf, length);
-		if (inb (STATUS (port)) & 0x01) {
+		//if (inb (STATUS (port)) & 0x01) {
+		if (readb (STATUS (port)) & 0x01) {
 			clear_epp_timeout (port);
 			return -EIO;
 		}
@@ -442,7 +462,8 @@
 	for (; written < length; written++) {
 		outb (*((char*)buf), EPPADDR (port));
 		buf++;
-		if (inb (STATUS (port)) & 0x01) {
+		//if (inb (STATUS (port)) & 0x01) {
+		if (readb (STATUS (port)) & 0x01) {
 			clear_epp_timeout (port);
 			break;
 		}
@@ -535,7 +556,8 @@
 
 	while (left) {
 		unsigned char byte;
-		unsigned char ecrval = inb (ECONTROL (port));
+		//unsigned char ecrval = inb (ECONTROL (port));
+		unsigned char ecrval = readb (ECONTROL (port));
 		int i = 0;
 
 		if (need_resched() && time_before (jiffies, expire))
@@ -562,7 +584,8 @@
 				printk (KERN_DEBUG "FIFO write timed out\n");
 				break;
 			}
-			ecrval = inb (ECONTROL (port));
+			//ecrval = inb (ECONTROL (port));
+			ecrval = readb (ECONTROL (port));
 			if (!(ecrval & (1<<2))) {
 				if (need_resched() &&
 				    time_before (jiffies, expire))
@@ -593,7 +616,8 @@
 			continue;
 		} else if (i++ < poll_for) {
 			udelay (10);
-			ecrval = inb (ECONTROL (port));
+			//ecrval = inb (ECONTROL (port));
+			ecrval = readb (ECONTROL (port));
 			goto poll;
 		}
 
@@ -689,7 +713,8 @@
 			break;
 		}
 		/* Is serviceIntr set? */
-		if (!(inb (ECONTROL (port)) & (1<<2))) {
+		//if (!(inb (ECONTROL (port)) & (1<<2))) {
+		if (!(readb (ECONTROL (port)) & (1<<2))) {
 			cond_resched();
 
 			goto false_alarm;
@@ -792,7 +817,8 @@
 
 		/* Adjust for the contents of the FIFO. */
 		for (written -= priv->fifo_depth; ; written++) {
-			if (inb (ECONTROL (port)) & 0x2) {
+			//if (inb (ECONTROL (port)) & 0x2) {
+			if (readb (ECONTROL (port)) & 0x2) {
 				/* Full up. */
 				break;
 			}
@@ -887,7 +913,8 @@
 
 		/* Adjust for the contents of the FIFO. */
 		for (written -= priv->fifo_depth; ; written++) {
-			if (inb (ECONTROL (port)) & 0x2) {
+			//if (inb (ECONTROL (port)) & 0x2) {
+			if (readb (ECONTROL (port)) & 0x2) {
 				/* Full up. */
 				break;
 			}
@@ -1619,19 +1646,27 @@
 
 	/* Do a simple read-write test to make sure the port exists. */
 	w = 0xc;
-	outb (w, CONTROL (pb));
+	//outb (w, CONTROL (pb));
+//printk("--mj:%s--1-\n",__func__);
+	writeb (w, CONTROL (pb));
 
+//printk("--mj:%s--2-\n",__func__);
 	/* Is there a control register that we can read from?  Some
 	 * ports don't allow reads, so read_control just returns a
 	 * software copy. Some ports _do_ allow reads, so bypass the
 	 * software copy here.  In addition, some bits aren't
 	 * writable. */
-	r = inb (CONTROL (pb));
+	//r = inb (CONTROL (pb));
+	r = readb (CONTROL (pb));
+//printk("--mj:%s--3-\n",__func__);
 	if ((r & 0xf) == w) {
 		w = 0xe;
-		outb (w, CONTROL (pb));
-		r = inb (CONTROL (pb));
-		outb (0xc, CONTROL (pb));
+		//outb (w, CONTROL (pb));
+		writeb (w, CONTROL (pb));
+		//r = inb (CONTROL (pb));
+		r = readb (CONTROL (pb));
+		//outb (0xc, CONTROL (pb));
+		writeb (0xc, CONTROL (pb));
 		if ((r & 0xf) == w)
 			return PARPORT_MODE_PCSPP;
 	}
@@ -1646,6 +1681,7 @@
 	 * this stage, so we expect back what we wrote. */
 	w = 0xaa;
 	parport_pc_write_data (pb, w);
+//printk("--mj:%s--4-\n",__func__);
 	r = parport_pc_read_data (pb);
 	if (r == w) {
 		w = 0x55;
@@ -1665,11 +1701,13 @@
 			pb->base);
 	}
 
+//printk("--mj:%s--5-\n",__func__);
 	/* It's possible that we can't read the control register or
 	 * the data register.  In that case just believe the user. */
 	if (user_specified)
 		return PARPORT_MODE_PCSPP;
 
+//printk("--mj:%s--6-\n",__func__);
 	return 0;
 }
 
@@ -1691,9 +1729,11 @@
 	struct parport_pc_private *priv = pb->private_data;
 	unsigned char r = 0xc;
 
-	outb (r, CONTROL (pb));
+	//outb (r, CONTROL (pb));
+	writeb (r, CONTROL (pb));
 	if ((inb (ECONTROL (pb)) & 0x3) == (r & 0x3)) {
-		outb (r ^ 0x2, CONTROL (pb)); /* Toggle bit 1 */
+		//outb (r ^ 0x2, CONTROL (pb)); /* Toggle bit 1 */
+		writeb (r ^ 0x2, CONTROL (pb)); /* Toggle bit 1 */
 
 		r = inb (CONTROL (pb));
 		if ((inb (ECONTROL (pb)) & 0x2) == (r & 0x2))
@@ -1708,7 +1748,8 @@
 		goto no_reg;
 
 	priv->ecr = 1;
-	outb (0xc, CONTROL (pb));
+	//outb (0xc, CONTROL (pb));
+	writeb (0xc, CONTROL (pb));
 	
 	/* Go to mode 000 */
 	frob_set_mode (pb, ECR_SPP);
@@ -1716,7 +1757,8 @@
 	return 1;
 
  no_reg:
-	outb (0xc, CONTROL (pb));
+	//outb (0xc, CONTROL (pb));
+	writeb (0xc, CONTROL (pb));
 	return 0; 
 }
 
@@ -1748,11 +1790,13 @@
 	parport_pc_data_reverse (pb);
 	
 	parport_pc_write_data(pb, 0x55);
+printk("-supported-1-p->base:%lx---\n",pb->base);
 	if (parport_pc_read_data(pb) != 0x55) ok++;
 
 	parport_pc_write_data(pb, 0xaa);
 	if (parport_pc_read_data(pb) != 0xaa) ok++;
 
+printk("-supported-2-p->base:%lx---\n",pb->base);
 	/* cancel input mode */
 	parport_pc_data_forward (pb);
 
@@ -1784,7 +1828,8 @@
 	ECR_WRITE (pb, ECR_SPP << 5); /* Reset FIFO */
 	ECR_WRITE (pb, ECR_TST << 5); /* TEST FIFO */
 	for (i=0; i < 1024 && !(inb (ECONTROL (pb)) & 0x02); i++)
-		outb (0xaa, FIFO (pb));
+		//outb (0xaa, FIFO (pb));
+		writeb (0xaa, FIFO (pb));
 
 	/*
 	 * Using LGS chipset it uses ECR register, but
@@ -1827,7 +1872,8 @@
 	frob_econtrol (pb, 1<<2, 1<<2);
 	frob_econtrol (pb, 1<<2, 0);
 	for (i = 1; i <= priv->fifo_depth; i++) {
-		outb (0xaa, FIFO (pb));
+		//outb (0xaa, FIFO (pb));
+		writeb (0xaa, FIFO (pb));
 		if (inb (ECONTROL (pb)) & (1<<2))
 			break;
 	}
@@ -1970,7 +2016,8 @@
 	oecr = inb (ECONTROL (pb));
 	/* Search for SMC style EPP+ECP mode */
 	ECR_WRITE (pb, 0x80);
-	outb (0x04, CONTROL (pb));
+	//outb (0x04, CONTROL (pb));
+	writeb (0x04, CONTROL (pb));
 	result = parport_EPP_supported(pb);
 
 	ECR_WRITE (pb, oecr);
@@ -2032,7 +2079,8 @@
 
 	/* If Full FIFO sure that writeIntrThreshold is generated */
 	for (i=0; i < 1024 && !(inb (ECONTROL (pb)) & 0x02) ; i++) 
-		outb (0xaa, FIFO (pb));
+		//outb (0xaa, FIFO (pb));
+		writeb (0xaa, FIFO (pb));
 		
 	pb->irq = probe_irq_off(irqs);
 	ECR_WRITE (pb, ECR_SPP << 5);
@@ -2184,6 +2232,7 @@
 	struct resource	*EPP_res = NULL;
 	struct platform_device *pdev = NULL;
 
+//printk("--mj:%s--1-\n",__func__);
 	if (!dev) {
 		/* We need a physical device to attach to, but none was
 		 * provided. Create our own. */
@@ -2198,19 +2247,27 @@
 	if (!ops)
 		goto out1;
 
+//printk("--mj:%s--out1-\n",__func__);
+//printk("---p->base:%lx---",p->base);
 	priv = kmalloc (sizeof (struct parport_pc_private), GFP_KERNEL);
 	if (!priv)
 		goto out2;
 
+///printk("---p->base:%lx---",p->base);
 	/* a misnomer, actually - it's allocate and reserve parport number */
 	p = parport_register_port(base, irq, dma, ops);
 	if (!p)
 		goto out3;
 
-	base_res = request_region(base, 3, p->name);
+//printk("--mj:%s--out3-\n",__func__);
+//printk("---p->base:%lx---",p->base);
+#if 0
+	//base_res = request_region(base, 3, p->name);
+	base_res = request_mem_region(base, 3, p->name);
 	if (!base_res)
-		goto out4;
-
+	//	goto out4;
+#endif 
+//printk("--mj:%s--out4-\n",__func__);
 	memcpy(ops, &parport_pc_ops, sizeof (struct parport_operations));
 	priv->ctr = 0xc;
 	priv->ctr_writable = ~0x10;
@@ -2225,19 +2282,24 @@
 	p->base_hi = base_hi;
 	p->modes = PARPORT_MODE_PCSPP | PARPORT_MODE_SAFEININT;
 	p->private_data = priv;
-
+//printk("--mj:%s--2-\n",__func__);
+//printk("---p->base:%lx---",p->base);
 	if (base_hi) {
 		ECR_res = request_region(base_hi, 3, p->name);
 		if (ECR_res)
 			parport_ECR_present(p);
 	}
 
+//printk("--mj:%s--3-\n",__func__);
+//printk("---p->base:%lx---",p->base);
+#if 0
 	if (base != 0x3bc) {
 		EPP_res = request_region(base+0x3, 5, p->name);
 		if (EPP_res)
 			if (!parport_EPP_supported(p))
 				parport_ECPEPP_supported(p);
 	}
+#endif
 	if (!parport_SPP_supported (p))
 		/* No port. */
 		goto out5;
@@ -2269,6 +2331,7 @@
 			parport_dma_probe(p);
 		}
 	}
+printk("--1-p->base:%lx---",p->base);
 	if (p->dma == PARPORT_DMA_AUTO) /* To use DMA, giving the irq
                                            is mandatory (see above) */
 		p->dma = PARPORT_DMA_NONE;
@@ -2311,6 +2374,7 @@
 	printk ("(,...)");
 #endif /* CONFIG_PARPORT_1284 */
 	printk("]\n");
+printk("-2--p->base:%lx---",p->base);
 	if (probedirq != PARPORT_IRQ_NONE) 
 		printk(KERN_INFO "%s: irq %d detected\n", p->name, probedirq);
 
@@ -2370,6 +2434,8 @@
 		 */
 		ECR_WRITE (p, 0x34);
 
+//printk("--mj:%s--4-\n",__func__);
+//printk("-4-p->base:%lx---",p->base);
 	parport_pc_write_data(p, 0);
 	parport_pc_data_forward (p);
 
@@ -3173,13 +3239,17 @@
 parport_pc_find_isa_ports (int autoirq, int autodma)
 {
 	int count = 0;
-
+//printk("--mj:%s--partport_pc_probe55-\n",__func__);
+#if 0
 	if (parport_pc_probe_port(0x3bc, 0x7bc, autoirq, autodma, NULL))
 		count++;
 	if (parport_pc_probe_port(0x378, 0x778, autoirq, autodma, NULL))
 		count++;
 	if (parport_pc_probe_port(0x278, 0x678, autoirq, autodma, NULL))
 		count++;
+#endif
+	if (parport_pc_probe_port(0xffffffffbe0000e0, 0x0, autoirq, autodma, NULL))
+		count++;
 
 	return count;
 }
@@ -3448,6 +3518,7 @@
 
 	if (parse_parport_params())
 		return -EINVAL;
+	printk("----1----\n");
 
 	err = platform_driver_register(&parport_pc_platform_driver);
 	if (err)
@@ -3466,6 +3537,7 @@
 						  irqval[i], dmaval[i], NULL);
 		}
 	} else
+	printk("----2----\n");
 		parport_pc_find_ports (irqval[0], dmaval[0]);
 
 	return 0;
@@ -3491,9 +3563,284 @@
 		parport_pc_unregister_port(port);
 	}
 }
+#if 0 //mj add 
+/* S3C2410 ¿ª·¢°åÀ©Õ¹²¢¿Ú16C552µÄlow-levelÇý¶¯
+ *
+ * Author: Allen He<wingwind@sina.com>
+ *
+ *
+ *
+ * The extended parallel port provides one port at a fixed address
+ * with 8 output data lines (D0 - D7), 1 output control line (STROBE)
+ * and 1 input status line (BUSY) able to cause an interrupt.
+ */
+
+//#define MODULE  //HYL Add 
+
+
+//#include <asm/irq.h>
+//#include <asm/hardware.h>
+//#include <asm/arch/cpu_s3c2410.h>
+#include <linux/parport_2410_16c552.h>
+#define INT_EINT8_23 34
+static struct parport *this_port = NULL;
+
+ unsigned char
+parport_16c552_read_data(struct parport *p)
+{
+	unsigned long flags;
+	unsigned char data;
+
+	//save_flags(flags);
+	//local_save_flags(flags);
+	//cli();
+	data=*((unsigned char *)(p->base+0x0)) ;
+	//local_restore_flags(flags);
+printk("-----------------hah--mj:%s-data:%08x--\n",__func__,data);
+	return data;
+}
+
+static void
+parport_16c552_write_data(struct parport *p, unsigned char data)
+{
+	unsigned long flags;
+parport_16c552_read_data(p);
+printk("-----------------hah--mj:%s---%08x---%02x\n",__func__,(p->base),data);
+	//save_flags(flags);
+	//cli();
+	*((unsigned char *)(p->base+0x0))=data;
+	//restore_flags(flags);
+	return;
+}
+
+
+
+unsigned char parport_16c552_read_control(struct parport *p)
+{
+	unsigned long flags;
+	unsigned char control = 0;
+
+	//save_flags(flags);
+	//cli();
+	control=*((unsigned char *)(p->base+0x2));
+	//restore_flags(flags);
+	return control;
+
+}
+
+void parport_16c552_write_control(struct parport *p, unsigned char d)
+{
+
+	unsigned long flags;
+
+	//save_flags(flags);
+	//cli();
+	*((unsigned char *)(p->base+0x2))=d;
+	//restore_flags(flags);
+	return;
+}
+
+static unsigned char
+parport_16c552_frob_control(struct parport *p, unsigned char mask,
+			   unsigned char val)
+{
+	unsigned char old = parport_16c552_read_control(p);
+	parport_16c552_write_control(p, (old & ~mask) ^ val);
+	return old;
+}
+
+static unsigned char
+parport_16c552_read_status(struct parport *p)
+{
+printk("-----------------hah--mj:%s-data:%08x--\n",__func__,*((unsigned char *)(p->base+0x1)));
+	return (*((unsigned char *)(p->base+0x1)));
+}
+
+static void
+parport_16c552_init_state(struct pardevice *d, struct parport_state *s)
+{
+}
+
+static void
+parport_16c552_save_state(struct parport *p, struct parport_state *s)
+{
+}
+
+static void
+parport_16c552_restore_state(struct parport *p, struct parport_state *s)
+{
+}
+
+static void
+//parport_16c552_interrupt(int irq, void *dev_id, struct pt_regs *regs)
+parport_16c552_interrupt(void *dev_id)
+{
+//printk("-----------------hah--mj:%s---\n",__func__);
+	parport_generic_irq((struct parport *) dev_id);
+}
+
+static void
+parport_16c552_enable_irq(struct parport *p)
+{
+	enable_irq(INT_EINT8_23);
+}
+
+static void
+parport_16c552_disable_irq(struct parport *p)
+{
+	disable_irq(INT_EINT8_23);
+}
+
+static void
+parport_16c552_data_forward(struct parport *p)
+{
+	unsigned long flags;
+	////save_flags(flags);
+	//cli();
+	parport_16c552_frob_control(p,0x20,0x0);
+	//restore_flags(flags);
+}
+
+static void
+parport_16c552_data_reverse(struct parport *p)
+{
+	unsigned long flags;
+	//save_flags(flags);
+	//cli();
+	parport_16c552_frob_control(p,0x20,0x20);
+	//restore_flags(flags);
+}
+
+static void
+parport_16c552_inc_use_count(void)
+{
+//	MOD_INC_USE_COUNT;
+}
+
+static void
+parport_16c552_dec_use_count(void)
+{
+//	MOD_DEC_USE_COUNT;
+}
+
+static struct parport_operations parport_16c552_ops = {
+	parport_16c552_write_data,
+	parport_16c552_read_data,
+
+	parport_16c552_write_control,
+	parport_16c552_read_control,
+	parport_16c552_frob_control,
+
+	parport_16c552_read_status,
+
+	parport_16c552_enable_irq,
+	parport_16c552_disable_irq,
+
+	parport_16c552_data_forward,
+	parport_16c552_data_reverse,
+
+	parport_16c552_init_state,
+	parport_16c552_save_state,
+	parport_16c552_restore_state,
 
+	parport_16c552_inc_use_count,
+	parport_16c552_dec_use_count,
+
+	parport_ieee1284_epp_write_data,
+	parport_ieee1284_epp_read_data,
+	parport_ieee1284_epp_write_addr,
+	parport_ieee1284_epp_read_addr,
+
+	parport_ieee1284_ecp_write_data,
+	parport_ieee1284_ecp_read_data,
+	parport_ieee1284_ecp_write_addr,
+
+	parport_ieee1284_write_compat,
+	parport_ieee1284_read_nibble,
+	parport_ieee1284_read_byte,
+};
+
+
+int __init
+parport_16c552_init(void)
+{
+	struct parport *p;
+	unsigned long flags;
+
+//printk("--1---------------hah--mj:%s---\n",__func__);
+		/*·ÖÅäÄÚ´æÇøÓò*/
+		if(check_mem_region(brd_16c552_parportbase,0x3))//0x800 is a number defined as will.
+		{
+			printk("16c552 driver:memory already in use\n");
+			return -EBUSY;
+		}
+//printk("--2---------------hah--mj:%s---\n",__func__);
+		request_mem_region(brd_16c552_parportbase,0x3,"parport_16c552");		
+		
+///printk("--3---------------hah--mj:%s---\n",__func__);
+		p = parport_register_port(brd_16c552_parportbase,INT_EINT8_23,
+					  PARPORT_DMA_NONE,
+					  &parport_16c552_ops);
+//printk("--4---------------hah--mj:%s--,%s-,%08x\n",__func__,p->name,p);
+		if (!p)
+			return 0;
+		//if (request_irq(INT_EINT8_23, parport_16c552_interrupt,SA_SHIRQ, p->name, p)) {
+		if (request_irq(INT_EINT8_23, parport_16c552_interrupt,0, p->name, p)) {
+		//	parport_unregister_port (p);
+			return 0;
+		}
+	
+/*		// initialize the parport as a input device with disabling irq. 
+		save_flags(flags);
+		cli();
+		parport_16c552_frob_control(p,0x3f,0x24);		
+		restore_flags(flags);
+*/
+
+
+		this_port = p;
+		printk(KERN_INFO "%s: s3c2410 brd extended with 16c552, port using irq\n", p->name);
+		
+//printk("--5---------------hah--mj:%s---\n",__func__);
+		parport_proc_register(p);
+
+//printk("--6---------------hah--mj:%s---\n",__func__);
+		parport_announce_port (p);
+
+//printk("--0---------------hah--mj:%s---\n",__func__);
+		return 1;
+	
+}
+
+#if 0
+//#ifdef MODULE
+
+MODULE_AUTHOR("Wingwind");
+MODULE_DESCRIPTION("Parport Driver for 16c552 parellel Port");
+MODULE_SUPPORTED_DEVICE("S3c2410 extending 16c552 Parallel Port");
+MODULE_LICENSE("GPL");
+
+int
+init_module(void)
+{
+	return parport_16c552_init() ? 0 : -ENODEV;
+}
+
+void
+cleanup_module(void)
+{
+	if (this_port->irq != PARPORT_IRQ_NONE)
+		free_irq(IRQ_MFP_BUSY, this_port);
+	parport_proc_unregister(this_port);
+	parport_unregister_port(this_port);
+	release_mem_region(brd_16c552_parportbase,0x3,);	
+}
+#endif
+
+#endif
 MODULE_AUTHOR("Phil Blundell, Tim Waugh, others");
 MODULE_DESCRIPTION("PC-style parallel port driver");
 MODULE_LICENSE("GPL");
+//module_init(parport_16c552_init)
 module_init(parport_pc_init)
 module_exit(parport_pc_exit)
